# Databricks notebook source
# MAGIC %md
# MAGIC ## Read, transform, and write data
# MAGIC 
# MAGIC To work with data, you can use *Spark DataFrames*.

# COMMAND ----------

# reading data from mounted storage

# longer version
#df = spark.read.format('csv').option('header', True).load('/mnt/dp900sablob/extract/azuresql/SalesLT_ProductModel.txt');

#shorter version
#df = spark.read.option('header', True).csv('/mnt/dp900sablob/extract/azuresql/SalesLT_ProductModel.txt');

#shortest version
df = spark.read.csv('/mnt/dp900sablob/extract/azuresql/SalesLT_ProductModel.txt', header = True);

# COMMAND ----------

# 1. inferred schema
df = spark.read.csv('/mnt/dp900sablob/extract/azuresql/SalesLT_ProductModel.txt', header = True, inferSchema = True);

# COMMAND ----------

# 2. explicit schema
from pyspark.sql.types import *;

fileSchema = StructType([
  StructField("ProductModelID", IntegerType()),
  StructField("Name", StringType()),
  StructField("rowguid", StringType()),
  StructField("ModifiedDate", StringType())
]);

df = spark.read.csv('/mnt/dp900sablob/extract/azuresql/SalesLT_ProductModel.txt', header = True, schema = fileSchema);

# COMMAND ----------

#display first 1000 record
display(df);

# COMMAND ----------

#display first 100 record
display(df.limit(100));

# COMMAND ----------

# reading all source files
dfProductModel = spark.read.csv('/mnt/dp900sablob/extract/azuresql/SalesLT_ProductModel.txt', header = True, inferSchema = True);
dfProductCategory = spark.read.csv('/mnt/dp900sablob/extract/azuresql/SalesLT_ProductCategory.txt', header = True, inferSchema = True);
dfProductSubcategory = spark.read.csv('/mnt/dp900sablob/extract/azuresql/SalesLT_ProductCategory.txt', header = True, inferSchema = True);
dfProduct = spark.read.csv('/mnt/dp900sablob/extract/azuresql/SalesLT_Product.txt', header = True, inferSchema = True);

# COMMAND ----------

# performing the transformation
dfProductModel.createOrReplaceTempView('vwProductModel');
dfProductCategory.createOrReplaceTempView('vwProductCategory');
dfProductSubcategory.createOrReplaceTempView('vwProductSubcategory');
dfProduct.createOrReplaceTempView('vwProduct');

dfProductsInfo = spark.sql('''
SELECT 
  p.ProductID,
  p.Name,
  p.ProductNumber,
  p.Color,
  p.StandardCost,
  p.ListPrice,
  p.Size,
  p.Weight,
  pm.ProductModelID,  
  pm.Name AS ProductModelName,
  psc.ProductCategoryID AS ProductSubcategoryID,  
  psc.Name AS ProductSubcategoryName,
  pc.ProductCategoryID,
  pc.Name AS ProductCategoryName
FROM vwProduct AS p
INNER JOIN vwProductModel AS pm  ON p.ProductModelID = pm.ProductModelID
INNER JOIN vwProductCategory AS psc ON p.ProductCategoryID = psc.ProductCategoryID
INNER JOIN vwProductSubcategory AS pc ON psc.ParentProductCategoryID = pc.ProductCategoryID  
''');

# COMMAND ----------

# writing to disk (engine decides number of partitions)
dfProductsInfo.write.mode("overwrite").csv('/mnt/dp900sablob/transform/adb/products_info', header = True);

# COMMAND ----------

# writing to disk (fixed number of partitions)
dfProductsInfo.coalesce(1).write.mode("overwrite").csv('/mnt/dp900sablob/transform/adb/products_info', header = True);
